﻿using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using System.Threading.Tasks;

using TestIdentityServer.Models;

namespace TestIdentityServer.Controllers
{
    /// <summary>
    /// 인증 컨트롤러
    /// </summary>
    public class AuthController : Controller
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 사용자 관리자
        /// </summary>
        private UserManager<IdentityUser> userManager;

        /// <summary>
        /// 서명 관리자
        /// </summary>
        private SignInManager<IdentityUser> signInManager;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - AuthController(userManager, signInManager)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="userManager">사용자 관리자</param>
        /// <param name="signInManager">서명 관리자</param>
        public AuthController(UserManager<IdentityUser> userManager, SignInManager<IdentityUser> signInManager)
        {
            this.userManager   = userManager;
            this.signInManager = signInManager;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 로그인 페이지 처리하기 - Login(returnURL)

        /// <summary>
        /// 로그인 페이지 처리하기
        /// </summary>
        /// <param name="returnURL">반환 URL</param>
        /// <returns>액션 결과</returns>
        [HttpGet]
        public IActionResult Login(string returnURL)
        {
            return View(new LoginViewModel { ReturnURL = returnURL });
        }

        #endregion
        #region 로그인 페이지 처리하기 - Login(vm)

        /// <summary>
        /// 로그인 페이지 처리하기
        /// </summary>
        /// <param name="vm">로그인 뷰 모델</param>
        /// <returns>액션 결과</returns>
        [HttpPost]
        public async Task<IActionResult> Login(LoginViewModel vm)
        {
            Microsoft.AspNetCore.Identity.SignInResult result = await this.signInManager.PasswordSignInAsync(vm.UserName, vm.Password, false, false);

            if(result.Succeeded)
            {
                return Redirect(vm.ReturnURL);
            }

            return View();
        }

        #endregion

        #region 등록 페이지 처리하기 - Register(returnURL)

        /// <summary>
        /// 등록 페이지 처리하기
        /// </summary>
        /// <param name="returnURL">반환 URL</param>
        /// <returns>액션 결과</returns>
        [HttpGet]
        public IActionResult Register(string returnURL)
        {
            return View(new RegisterViewModel { ReturnURL = returnURL });
        }

        #endregion
        #region 등록 페이지 처리하기 - Register(vm)

        /// <summary>
        /// 등록 페이지 처리하기
        /// </summary>
        /// <param name="vm">등록 뷰 모델</param>
        /// <returns>액션 결과</returns>
        [HttpPost]
        public async Task<IActionResult> Register(RegisterViewModel vm)
        {
            if(!ModelState.IsValid)
            {
                return View(vm);
            }

            IdentityUser identityUser = new IdentityUser(vm.UserName);

            IdentityResult result = await this.userManager.CreateAsync(identityUser, vm.Password);

            if(result.Succeeded)
            {
                await this.signInManager.SignInAsync(identityUser, false);

                return Redirect(vm.ReturnURL);
            }

            return View();
        }

        #endregion
    }
}